@@ -24,17 +24,17 @@
 #include <kprotocolinfo.h>
 #include <ktemporaryfile.h>
 
+#include <kurl.h>
 #include <klocale.h>
 #include <kio/netaccess.h>
 #include <kio/job.h>
 #include <kio/copyjob.h>
 #include <kio/deletejob.h>
-#include <QDebug>
+#include <kdebug.h>
 #include <kcmdlineargs.h>
 #include <kconfiggroup.h>
 
 #include <QDir>
-#include <QUrl>
 #include <QFileInfo>
 #include <QVector>
 #include <kjobuidelegate.h>
@@ -47,7 +47,6 @@
 #include <kstandarddirs.h>
 #include <kio/chmodjob.h>
 #include <kio/directorysizejob.h>
-#include <QStandardPaths>
 
 // There are two ways to test encoding things:
 // * with utf8 filenames
@@ -79,7 +78,7 @@ Q_CONSTRUCTOR_FUNCTION(initLocale)
 
 QString TestTrash::homeTmpDir() const
 {
-    return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QString::fromLatin1("/testtrash/");
+    return KGlobal::dirs()->localkdedir() + QString::fromLatin1("testtrash/");
 }
 
 QString TestTrash::readOnlyDirPath() const
@@ -122,8 +121,8 @@ static void removeDirRecursive( const QString& dir )
     if ( QFile::exists( dir ) ) {
 
         // Make it work even with readonly dirs, like trashReadOnlyDirFromHome() creates
-        QUrl u = QUrl::fromLocalFile( dir );
-        //qDebug() << "chmod +0200 on" << u;
+        KUrl u = KUrl::fromPath( dir );
+        //kDebug() << "chmod +0200 on" << u;
         KFileItem fileItem(u, QString::fromLatin1("inode/directory"), KFileItem::Unknown);
         KFileItemList fileItemList;
         fileItemList.append( fileItem );
@@ -132,7 +131,7 @@ static void removeDirRecursive( const QString& dir )
 
         KIO::Job* delJob = KIO::del(u, KIO::HideProgressInfo);
         if (!KIO::NetAccess::synchronousRun(delJob, 0))
-            qFatal("Couldn't delete %s", qPrintable(dir));
+            kFatal() << "Couldn't delete " << dir ;
     }
 }
 
@@ -142,8 +141,8 @@ void TestTrash::initTestCase()
     setenv( "KDE_FORK_SLAVES", "yes", true );
 
 
-    m_trashDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QString::fromLatin1("/Trash");
-    qDebug() << "setup: using trash directory " << m_trashDir;
+    m_trashDir = KGlobal::dirs()->localxdgdatadir() + QString::fromLatin1("Trash");
+    kDebug() << "setup: using trash directory " << m_trashDir;
 
     // Look for another writable partition than $HOME (not mandatory)
     TrashImpl impl;
@@ -169,7 +168,7 @@ void TestTrash::initTestCase()
                 if (topdir == QLatin1String("/tmp/")) {
                     m_tmpIsWritablePartition = true;
                     m_tmpTrashId = it.key();
-                    qDebug() << "/tmp is on its own partition (trashid=" << m_tmpTrashId << "), some tests will be skipped";
+                    kDebug() << "/tmp is on its own partition (trashid=" << m_tmpTrashId << "), some tests will be skipped";
                     removeFile( it.value(), QString::fromLatin1("/info/fileFromOther.trashinfo") );
                     removeFile( it.value(), QString::fromLatin1("/files/fileFromOther") );
                     removeFile( it.value(), QString::fromLatin1("/info/symlinkFromOther.trashinfo") );
@@ -192,7 +191,7 @@ void TestTrash::initTestCase()
             m_otherPartitionTopDir = topdir;
             m_otherPartitionTrashDir = trashdir;
             m_otherPartitionId = *it;
-            qDebug() << "OK, found another writable partition: topDir=" << m_otherPartitionTopDir
+            kDebug() << "OK, found another writable partition: topDir=" << m_otherPartitionTopDir
                       << " trashDir=" << m_otherPartitionTrashDir << " id=" << m_otherPartitionId << endl;
             break;
         }
@@ -200,23 +199,23 @@ void TestTrash::initTestCase()
     // Check that m_trashDir got listed
     QVERIFY( foundTrashDir );
     if ( m_otherPartitionTrashDir.isEmpty() )
-        qWarning() << "No writable partition other than $HOME found, some tests will be skipped" ;
+        kWarning() << "No writable partition other than $HOME found, some tests will be skipped" ;
 
     // Start with a clean base dir
-    qDebug() << "initial cleanup";
+    kDebug() << "initial cleanup";
     removeDirRecursive( homeTmpDir() );
     removeDirRecursive( otherTmpDir() );
 
     QDir dir; // TT: why not a static method?
     bool ok = dir.mkdir( homeTmpDir() );
     if ( !ok )
-        qFatal("Couldn't create directory: %s", qPrintable(homeTmpDir()));
+        kFatal() << "Couldn't create " << homeTmpDir() ;
     ok = dir.mkdir( otherTmpDir() );
     if ( !ok )
-        qFatal("Couldn't create directory: %s", qPrintable(otherTmpDir()));
+        kFatal() << "Couldn't create " << otherTmpDir() ;
 
     // Start with a clean trash too
-    qDebug() << "removing trash dir";
+    kDebug() << "removing trash dir";
     removeDirRecursive( m_trashDir );
 }
 
@@ -230,7 +229,7 @@ void TestTrash::cleanupTestCase()
 
 void TestTrash::urlTestFile()
 {
-    const QUrl url = TrashImpl::makeURL(1, QString::fromLatin1("fileId"), QString());
+    const KUrl url = TrashImpl::makeURL(1, QString::fromLatin1("fileId"), QString());
     QCOMPARE( url.url(), QString::fromLatin1("trash:/1-fileId"));
 
     int trashId;
@@ -245,7 +244,7 @@ void TestTrash::urlTestFile()
 
 void TestTrash::urlTestDirectory()
 {
-    const QUrl url = TrashImpl::makeURL( 1, QString::fromLatin1("fileId"), QString::fromLatin1("subfile") );
+    const KUrl url = TrashImpl::makeURL( 1, QString::fromLatin1("fileId"), QString::fromLatin1("subfile") );
     QCOMPARE( url.url(), QString::fromLatin1( "trash:/1-fileId/subfile" ) );
 
     int trashId;
@@ -260,7 +259,7 @@ void TestTrash::urlTestDirectory()
 
 void TestTrash::urlTestSubDirectory()
 {
-    const QUrl url = TrashImpl::makeURL(1, QString::fromLatin1("fileId"), QString::fromLatin1("subfile/foobar"));
+    const KUrl url = TrashImpl::makeURL(1, QString::fromLatin1("fileId"), QString::fromLatin1("subfile/foobar"));
     QCOMPARE(url.url(), QString::fromLatin1("trash:/1-fileId/subfile/foobar"));
 
     int trashId;
@@ -275,14 +274,14 @@ void TestTrash::urlTestSubDirectory()
 
 static void checkInfoFile( const QString& infoPath, const QString& origFilePath )
 {
-    qDebug() << infoPath;
+    kDebug() << infoPath;
     QFileInfo info( infoPath );
     QVERIFY( info.exists() );
     QVERIFY( info.isFile() );
     KConfig infoFile( info.absoluteFilePath() );
     KConfigGroup group = infoFile.group( "Trash Info" );
     if ( !group.exists() )
-        qFatal("no Trash Info group in %s", qPrintable(info.absoluteFilePath()));
+        kFatal() << "no Trash Info group in " << info.absoluteFilePath() ;
     const QString origPath = group.readEntry( "Path" );
     QVERIFY(!origPath.isEmpty());
     QVERIFY(origPath == QString::fromLatin1(QUrl::toPercentEncoding(origFilePath, "/")));
@@ -300,7 +299,7 @@ static void createTestFile( const QString& path )
 {
     QFile f( path );
     if ( !f.open( QIODevice::WriteOnly ) )
-        qFatal("Can't create %s", qPrintable(path));
+        kFatal() << "Can't create " << path ;
     f.write( "Hello world\n", 12 );
     f.close();
     QVERIFY( QFile::exists( path ) );
@@ -311,17 +310,18 @@ void TestTrash::trashFile( const QString& origFilePath, const QString& fileId )
     // setup
     if ( !QFile::exists( origFilePath ) )
         createTestFile( origFilePath );
-    QUrl u = QUrl::fromLocalFile( origFilePath );
+    KUrl u;
+    u.setPath( origFilePath );
 
     // test
-    KIO::Job* job = KIO::move( u, QUrl("trash:/"), KIO::HideProgressInfo );
+    KIO::Job* job = KIO::move( u, KUrl("trash:/"), KIO::HideProgressInfo );
     QMap<QString, QString> metaData;
     bool ok = KIO::NetAccess::synchronousRun( job, 0, 0, 0, &metaData );
     if ( !ok )
-        qCritical() << "moving " << u << " to trash failed with error " << KIO::NetAccess::lastError() << " " << KIO::NetAccess::lastErrorString() << endl;
+        kError() << "moving " << u << " to trash failed with error " << KIO::NetAccess::lastError() << " " << KIO::NetAccess::lastErrorString() << endl;
     QVERIFY( ok );
     if (origFilePath.startsWith(QLatin1String("/tmp")) && m_tmpIsWritablePartition) {
-        qDebug() << " TESTS SKIPPED";
+        kDebug() << " TESTS SKIPPED";
     } else {
         checkInfoFile(m_trashDir + QString::fromLatin1("/info/") + fileId + QString::fromLatin1(".trashinfo"), origFilePath);
 
@@ -339,10 +339,10 @@ void TestTrash::trashFile( const QString& origFilePath, const QString& fileId )
     for ( ; it != metaData.constEnd() ; ++it ) {
         if (it.key().startsWith(QLatin1String("trashURL"))) {
             const QString origPath = it.key().mid( 9 );
-            QUrl trashURL( it.value() );
-            qDebug() << trashURL;
+            KUrl trashURL( it.value() );
+            kDebug() << trashURL;
             QVERIFY(!trashURL.isEmpty());
-            QVERIFY(trashURL.scheme() == QLatin1String("trash"));
+            QVERIFY(trashURL.protocol() == QLatin1String("trash"));
             int trashId = 0;
             if (origFilePath.startsWith(QLatin1String("/tmp")) && m_tmpIsWritablePartition)
                 trashId = m_tmpTrashId;
@@ -399,7 +399,7 @@ void TestTrash::trashFileFromOther()
 void TestTrash::trashFileIntoOtherPartition()
 {
     if ( m_otherPartitionTrashDir.isEmpty() ) {
-        qDebug() << " - SKIPPED";
+        kDebug() << " - SKIPPED";
         return;
     }
     const QString fileName = QString::fromLatin1("testtrash-file");
@@ -412,10 +412,11 @@ void TestTrash::trashFileIntoOtherPartition()
     // setup
     if ( !QFile::exists( origFilePath ) )
         createTestFile( origFilePath );
-    QUrl u = QUrl::fromLocalFile( origFilePath );
+    KUrl u;
+    u.setPath( origFilePath );
 
     // test
-    KIO::Job* job = KIO::move( u, QUrl("trash:/"), KIO::HideProgressInfo );
+    KIO::Job* job = KIO::move( u, KUrl("trash:/"), KIO::HideProgressInfo );
     QMap<QString, QString> metaData;
     bool ok = KIO::NetAccess::synchronousRun( job, 0, 0, 0, &metaData );
     QVERIFY( ok );
@@ -435,10 +436,10 @@ void TestTrash::trashFileIntoOtherPartition()
     for ( ; it != metaData.constEnd() ; ++it ) {
         if (it.key().startsWith( QLatin1String("trashURL"))) {
             const QString origPath = it.key().mid( 9 );
-            QUrl trashURL( it.value() );
-            qDebug() << trashURL;
+            KUrl trashURL( it.value() );
+            kDebug() << trashURL;
             QVERIFY( !trashURL.isEmpty() );
-            QVERIFY(trashURL.scheme() == QLatin1String("trash"));
+            QVERIFY(trashURL.protocol() == QLatin1String("trash"));
             QVERIFY(trashURL.path() == QString::fromLatin1("/%1-%2").arg( m_otherPartitionId ).arg(fileId));
             found = true;
         }
@@ -448,15 +449,14 @@ void TestTrash::trashFileIntoOtherPartition()
 
 void TestTrash::trashFileOwnedByRoot()
 {
-    QUrl u( "file:///etc/passwd" );
+    KUrl u( "/etc/passwd" );
     const QString fileId = QString::fromLatin1("passwd");
 
-    KIO::CopyJob* job = KIO::move( u, QUrl("trash:/"), KIO::HideProgressInfo );
+    KIO::CopyJob* job = KIO::move( u, KUrl("trash:/"), KIO::HideProgressInfo );
     job->setUiDelegate(0); // no skip dialog, thanks
     QMap<QString, QString> metaData;
     bool ok = KIO::NetAccess::synchronousRun( job, 0, 0, 0, &metaData );
     QVERIFY( !ok );
-
     QVERIFY( KIO::NetAccess::lastError() == KIO::ERR_ACCESS_DENIED );
     const QString infoPath(m_trashDir + QString::fromLatin1("/info/") + fileId + QString::fromLatin1(".trashinfo"));
     QVERIFY( !QFile::exists( infoPath ) );
@@ -473,14 +473,15 @@ void TestTrash::trashSymlink( const QString& origFilePath, const QString& fileId
     const char* target = broken ? "/nonexistent" : "/tmp";
     bool ok = ::symlink( target, QFile::encodeName( origFilePath ) ) == 0;
     QVERIFY( ok );
-    QUrl u = QUrl::fromLocalFile( origFilePath );
+    KUrl u;
+    u.setPath( origFilePath );
 
     // test
-    KIO::Job* job = KIO::move( u, QUrl("trash:/"), KIO::HideProgressInfo );
+    KIO::Job* job = KIO::move( u, KUrl("trash:/"), KIO::HideProgressInfo );
     ok = job->exec();
     QVERIFY( ok );
     if (origFilePath.startsWith(QLatin1String("/tmp")) && m_tmpIsWritablePartition) {
-        qDebug() << " TESTS SKIPPED";
+        kDebug() << " TESTS SKIPPED";
         return;
     }
     checkInfoFile(m_trashDir + QString::fromLatin1("/info/") + fileId + QString::fromLatin1(".trashinfo"), origFilePath);
@@ -511,7 +512,7 @@ void TestTrash::trashBrokenSymlinkFromHome()
 
 void TestTrash::trashDirectory( const QString& origPath, const QString& fileId )
 {
-    qDebug() << fileId;
+    kDebug() << fileId;
     // setup
     if ( !QFileInfo( origPath ).exists() ) {
         QDir dir;
@@ -521,13 +522,13 @@ void TestTrash::trashDirectory( const QString& origPath, const QString& fileId )
     createTestFile(origPath + QString::fromLatin1("/testfile"));
     QVERIFY(QDir().mkdir(origPath + QString::fromLatin1("/subdir")));
     createTestFile(origPath + QString::fromLatin1("/subdir/subfile"));
-    QUrl u = QUrl::fromLocalFile( origPath );
+    KUrl u; u.setPath( origPath );
 
     // test
-    KIO::Job* job = KIO::move( u, QUrl("trash:/"), KIO::HideProgressInfo );
+    KIO::Job* job = KIO::move( u, KUrl("trash:/"), KIO::HideProgressInfo );
     QVERIFY( job->exec() );
     if (origPath.startsWith(QLatin1String("/tmp")) && m_tmpIsWritablePartition) {
-        qDebug() << " TESTS SKIPPED";
+        kDebug() << " TESTS SKIPPED";
         return;
     }
     checkInfoFile(m_trashDir + QString::fromLatin1("/info/") + fileId + QString::fromLatin1(".trashinfo"), origPath);
@@ -540,6 +541,43 @@ void TestTrash::trashDirectory( const QString& origPath, const QString& fileId )
     QVERIFY( files.size() == 12 );
     QVERIFY( !QFile::exists( origPath ) );
     QVERIFY(QFile::exists(m_trashDir + QString::fromLatin1("/files/") + fileId + QString::fromLatin1("/subdir/subfile")));
+
+    QFile dirCache(m_trashDir + QString::fromLatin1("/directorysizes"));
+    QVERIFY2(dirCache.open(QIODevice::ReadOnly), qPrintable(dirCache.fileName()));
+    QByteArray lines;
+    bool found = false;
+    while (!dirCache.atEnd()) {
+        const QByteArray line = dirCache.readLine();
+        if (line.endsWith(' ' + QFile::encodeName(fileId).toPercentEncoding() + '\n')) {
+            QVERIFY(!found); // should be there only once!
+            found = true;
+        }
+        lines += line;
+    }
+    QVERIFY2(found, lines.constData());
+    //qDebug() << lines;
+
+    checkDirCacheValidity();
+}
+
+void TestTrash::checkDirCacheValidity()
+{
+    QFile dirCache(m_trashDir + QString::fromLatin1("/directorysizes"));
+    QVERIFY2(dirCache.open(QIODevice::ReadOnly), qPrintable(dirCache.fileName()));
+    QSet<QByteArray> seenDirs;
+    while (!dirCache.atEnd()) {
+        QByteArray line = dirCache.readLine();
+        QVERIFY(line.endsWith('\n'));
+        line.chop(1);
+        qDebug() << "LINE" << line;
+        const int lastSpace = line.lastIndexOf(' ');
+        const QByteArray dir = QByteArray::fromPercentEncoding(line.mid(lastSpace + 1));
+        QVERIFY2(!seenDirs.contains(dir), dir.constData());
+        seenDirs.insert(dir);
+        const QString localDir =m_trashDir + QString::fromLatin1("/files/") + QFile::decodeName(dir);
+        QVERIFY2(QFile::exists(localDir), qPrintable(localDir));
+        QVERIFY(QFileInfo(localDir).isDir());
+    }
 }
 
 void TestTrash::trashDirectoryFromHome()
@@ -582,14 +620,14 @@ void TestTrash::trashDirectoryFromOther()
 
 void TestTrash::tryRenameInsideTrash()
 {
-    qDebug() << " with file_move";
-    KIO::Job* job = KIO::file_move( QUrl("trash:/0-tryRenameInsideTrash"), QUrl("trash:/foobar"), -1, KIO::HideProgressInfo );
+    kDebug() << " with file_move";
+    KIO::Job* job = KIO::file_move( KUrl("trash:/0-tryRenameInsideTrash"), KUrl("trash:/foobar"), -1, KIO::HideProgressInfo );
     bool worked = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( !worked );
     QVERIFY( KIO::NetAccess::lastError() == KIO::ERR_CANNOT_RENAME );
 
-    qDebug() << " with move";
-    job = KIO::move( QUrl("trash:/0-tryRenameInsideTrash"), QUrl("trash:/foobar"), KIO::HideProgressInfo );
+    kDebug() << " with move";
+    job = KIO::move( KUrl("trash:/0-tryRenameInsideTrash"), KUrl("trash:/foobar"), KIO::HideProgressInfo );
     worked = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( !worked );
     QVERIFY( KIO::NetAccess::lastError() == KIO::ERR_CANNOT_RENAME );
@@ -598,7 +636,7 @@ void TestTrash::tryRenameInsideTrash()
 void TestTrash::delRootFile()
 {
     // test deleting a trashed file
-    KIO::Job* delJob = KIO::del(QUrl("trash:/0-fileFromHome"), KIO::HideProgressInfo);
+    KIO::Job* delJob = KIO::del(KUrl("trash:/0-fileFromHome"), KIO::HideProgressInfo);
     bool ok = KIO::NetAccess::synchronousRun(delJob, 0);
     QVERIFY( ok );
 
@@ -615,7 +653,7 @@ void TestTrash::delRootFile()
 void TestTrash::delFileInDirectory()
 {
     // test deleting a file inside a trashed directory -> not allowed
-    KIO::Job* delJob = KIO::del(QUrl("trash:/0-trashDirFromHome/testfile"), KIO::HideProgressInfo);
+    KIO::Job* delJob = KIO::del(KUrl("trash:/0-trashDirFromHome/testfile"), KIO::HideProgressInfo);
     bool ok = KIO::NetAccess::synchronousRun(delJob, 0);
     QVERIFY( !ok );
     QVERIFY( KIO::NetAccess::lastError() == KIO::ERR_ACCESS_DENIED );
@@ -631,7 +669,7 @@ void TestTrash::delFileInDirectory()
 void TestTrash::delDirectory()
 {
     // test deleting a trashed directory
-    KIO::Job* delJob = KIO::del(QUrl("trash:/0-trashDirFromHome"), KIO::HideProgressInfo);
+    KIO::Job* delJob = KIO::del(KUrl("trash:/0-trashDirFromHome"), KIO::HideProgressInfo);
     bool ok = KIO::NetAccess::synchronousRun(delJob, 0);
     QVERIFY( ok );
 
@@ -648,7 +686,7 @@ void TestTrash::delDirectory()
 }
 
 // KIO::NetAccess::stat() doesn't set HideProgressInfo - but it's not much work to do it ourselves:
-static bool MyNetAccess_stat(const QUrl& url, KIO::UDSEntry& entry)
+static bool MyNetAccess_stat(const KUrl& url, KIO::UDSEntry& entry)
 {
     KIO::StatJob * statJob = KIO::stat( url, KIO::HideProgressInfo );
     bool ok = KIO::NetAccess::synchronousRun(statJob, 0);
@@ -656,7 +694,7 @@ static bool MyNetAccess_stat(const QUrl& url, KIO::UDSEntry& entry)
         entry = statJob->statResult();
     return ok;
 }
-static bool MyNetAccess_exists(const QUrl& url)
+static bool MyNetAccess_exists(const KUrl& url)
 {
     KIO::UDSEntry dummy;
     return MyNetAccess_stat(url, dummy);
@@ -664,7 +702,7 @@ static bool MyNetAccess_exists(const QUrl& url)
 
 void TestTrash::statRoot()
 {
-    QUrl url( "trash:/" );
+    KUrl url( "trash:/" );
     KIO::UDSEntry entry;
     bool ok = MyNetAccess_stat( url, entry );
     QVERIFY( ok );
@@ -679,7 +717,7 @@ void TestTrash::statRoot()
 
 void TestTrash::statFileInRoot()
 {
-    QUrl url( "trash:/0-fileFromHome" );
+    KUrl url( "trash:/0-fileFromHome" );
     KIO::UDSEntry entry;
     bool ok = MyNetAccess_stat( url, entry );
     QVERIFY( ok );
@@ -695,7 +733,7 @@ void TestTrash::statFileInRoot()
 
 void TestTrash::statDirectoryInRoot()
 {
-    QUrl url( "trash:/0-trashDirFromHome" );
+    KUrl url( "trash:/0-trashDirFromHome" );
     KIO::UDSEntry entry;
     bool ok = MyNetAccess_stat( url, entry );
     QVERIFY( ok );
@@ -710,7 +748,7 @@ void TestTrash::statDirectoryInRoot()
 
 void TestTrash::statSymlinkInRoot()
 {
-    QUrl url( "trash:/0-symlinkFromHome" );
+    KUrl url( "trash:/0-symlinkFromHome" );
     KIO::UDSEntry entry;
     bool ok = MyNetAccess_stat( url, entry );
     QVERIFY( ok );
@@ -725,7 +763,7 @@ void TestTrash::statSymlinkInRoot()
 
 void TestTrash::statFileInDirectory()
 {
-    QUrl url( "trash:/0-trashDirFromHome/testfile" );
+    KUrl url( "trash:/0-trashDirFromHome/testfile" );
     KIO::UDSEntry entry;
     bool ok = MyNetAccess_stat( url, entry );
     QVERIFY( ok );
@@ -740,15 +778,16 @@ void TestTrash::statFileInDirectory()
 
 void TestTrash::copyFromTrash( const QString& fileId, const QString& destPath, const QString& relativePath )
 {
-    QUrl src(QString::fromLatin1("trash:/0-") + fileId);
+    KUrl src(QString::fromLatin1("trash:/0-") + fileId);
     if ( !relativePath.isEmpty() )
-        src.setPath(src.path() + '/' + relativePath);
-    QUrl dest = QUrl::fromLocalFile( destPath );
+        src.addPath( relativePath );
+    KUrl dest;
+    dest.setPath( destPath );
 
     QVERIFY(MyNetAccess_exists(src));
 
     // A dnd would use copy(), but we use copyAs to ensure the final filename
-    //qDebug() << "copyAs:" << src << " -> " << dest;
+    //kDebug() << "copyAs:" << src << " -> " << dest;
     KIO::Job* job = KIO::copyAs( src, dest, KIO::HideProgressInfo );
     bool ok = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( ok );
@@ -799,10 +838,11 @@ void TestTrash::copySymlinkFromTrash()
 
 void TestTrash::moveFromTrash( const QString& fileId, const QString& destPath, const QString& relativePath )
 {
-    QUrl src( QString::fromLatin1("trash:/0-") + fileId );
+    KUrl src( QString::fromLatin1("trash:/0-") + fileId );
     if ( !relativePath.isEmpty() )
-        src.setPath(src.path() + '/' + relativePath);
-    QUrl dest = QUrl::fromLocalFile( destPath );
+        src.addPath( relativePath );
+    KUrl dest;
+    dest.setPath( destPath );
 
     QVERIFY(MyNetAccess_exists(src));
 
@@ -844,6 +884,7 @@ void TestTrash::moveDirectoryFromTrash()
     const QString destPath = otherTmpDir() + "trashDirFromHome_restored";
     moveFromTrash( "trashDirFromHome", destPath );
     QVERIFY( QFileInfo( destPath ).isDir() );
+    checkDirCacheValidity();
 
     // trash it again, we'll need it later
     QString dirName = "trashDirFromHome";
@@ -852,7 +893,7 @@ void TestTrash::moveDirectoryFromTrash()
 
 void TestTrash::trashDirectoryOwnedByRoot()
 {
-    QUrl u("file:///");;
+    KUrl u;
     if ( QFile::exists( "/etc/cups" ) )
         u.setPath( "/etc/cups" );
     else if ( QFile::exists( "/boot" ) )
@@ -860,9 +901,9 @@ void TestTrash::trashDirectoryOwnedByRoot()
     else
         u.setPath( "/etc" );
     const QString fileId = u.path();
-    qDebug() << "fileId=" << fileId;
+    kDebug() << "fileId=" << fileId;
 
-    KIO::CopyJob* job = KIO::move( u, QUrl("trash:/"), KIO::HideProgressInfo );
+    KIO::CopyJob* job = KIO::move( u, KUrl("trash:/"), KIO::HideProgressInfo );
     job->setUiDelegate(0); // no skip dialog, thanks
     QMap<QString, QString> metaData;
     bool ok = KIO::NetAccess::synchronousRun( job, 0, 0, 0, &metaData );
@@ -890,16 +931,16 @@ void TestTrash::moveSymlinkFromTrash()
 void TestTrash::getFile()
 {
     const QString fileId = "fileFromHome 1";
-    const QUrl url = TrashImpl::makeURL( 0, fileId, QString() );
+    const KUrl url = TrashImpl::makeURL( 0, fileId, QString() );
 
     KTemporaryFile tmpFile;
     QVERIFY(tmpFile.open());
     const QString tmpFilePath = tmpFile.fileName();
 
-    KIO::Job* getJob = KIO::file_copy(url, QUrl::fromLocalFile(tmpFilePath), -1, KIO::Overwrite | KIO::HideProgressInfo);
+    KIO::Job* getJob = KIO::file_copy(url, KUrl(tmpFilePath), -1, KIO::Overwrite | KIO::HideProgressInfo);
     bool ok = KIO::NetAccess::synchronousRun(getJob, 0);
     if (!ok) {
-        qDebug() << getJob->errorString();
+        kDebug() << getJob->errorString();
     }
     QVERIFY( ok );
     // Don't use tmpFile.close()+tmpFile.open() here, the size would still be 0 in the QTemporaryFile object
@@ -914,7 +955,7 @@ void TestTrash::getFile()
 void TestTrash::restoreFile()
 {
     const QString fileId = "fileFromHome 1";
-    const QUrl url = TrashImpl::makeURL( 0, fileId, QString() );
+    const KUrl url = TrashImpl::makeURL( 0, fileId, QString() );
     const QString infoFile( m_trashDir + "/info/" + fileId + ".trashinfo" );
     const QString filesItem( m_trashDir + "/files/" + fileId );
 
@@ -940,7 +981,7 @@ void TestTrash::restoreFileFromSubDir()
     const QString fileId = "trashDirFromHome 1/testfile";
     QVERIFY( !QFile::exists( homeTmpDir() + "trashDirFromHome 1" ) );
 
-    const QUrl url = TrashImpl::makeURL( 0, fileId, QString() );
+    const KUrl url = TrashImpl::makeURL( 0, fileId, QString() );
     const QString infoFile( m_trashDir + "/info/trashDirFromHome 1.trashinfo" );
     const QString filesItem( m_trashDir + "/files/trashDirFromHome 1/testfile" );
 
@@ -969,12 +1010,12 @@ void TestTrash::restoreFileToDeletedDirectory()
     removeFile( m_trashDir, "/files/fileFromHome" );
     trashFileFromHome();
     // Delete orig dir
-    KIO::Job* delJob = KIO::del(QUrl::fromLocalFile(homeTmpDir()), KIO::HideProgressInfo);
+    KIO::Job* delJob = KIO::del(KUrl(homeTmpDir()), KIO::HideProgressInfo);
     bool delOK = KIO::NetAccess::synchronousRun(delJob, 0);
     QVERIFY( delOK );
 
     const QString fileId = "fileFromHome";
-    const QUrl url = TrashImpl::makeURL( 0, fileId, QString() );
+    const KUrl url = TrashImpl::makeURL( 0, fileId, QString() );
     const QString infoFile( m_trashDir + "/info/" + fileId + ".trashinfo" );
     const QString filesItem( m_trashDir + "/files/" + fileId );
 
@@ -1003,16 +1044,16 @@ void TestTrash::listRootDir()
     m_entryCount = 0;
     m_listResult.clear();
     m_displayNameListResult.clear();
-    KIO::ListJob* job = KIO::listDir( QUrl("trash:/"), KIO::HideProgressInfo );
+    KIO::ListJob* job = KIO::listDir( KUrl("trash:/"), KIO::HideProgressInfo );
     connect( job, SIGNAL( entries( KIO::Job*, const KIO::UDSEntryList& ) ),
              SLOT( slotEntries( KIO::Job*, const KIO::UDSEntryList& ) ) );
     bool ok = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( ok );
-    qDebug() << "listDir done - m_entryCount=" << m_entryCount;
+    kDebug() << "listDir done - m_entryCount=" << m_entryCount;
     QVERIFY( m_entryCount > 1 );
 
-    //qDebug() << m_listResult;
-    //qDebug() << m_displayNameListResult;
+    //kDebug() << m_listResult;
+    //kDebug() << m_displayNameListResult;
     QCOMPARE(m_listResult.count( "." ), 1); // found it, and only once
     QCOMPARE(m_displayNameListResult.count( "fileFromHome" ), 1);
     QCOMPARE(m_displayNameListResult.count( "fileFromHome 1" ), 1);
@@ -1023,16 +1064,16 @@ void TestTrash::listRecursiveRootDir()
     m_entryCount = 0;
     m_listResult.clear();
     m_displayNameListResult.clear();
-    KIO::ListJob* job = KIO::listRecursive( QUrl("trash:/"), KIO::HideProgressInfo );
+    KIO::ListJob* job = KIO::listRecursive( KUrl("trash:/"), KIO::HideProgressInfo );
     connect( job, SIGNAL( entries( KIO::Job*, const KIO::UDSEntryList& ) ),
              SLOT( slotEntries( KIO::Job*, const KIO::UDSEntryList& ) ) );
     bool ok = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( ok );
-    qDebug() << "listDir done - m_entryCount=" << m_entryCount;
+    kDebug() << "listDir done - m_entryCount=" << m_entryCount;
     QVERIFY( m_entryCount > 1 );
 
-    qDebug() << m_listResult;
-    qDebug() << m_displayNameListResult;
+    kDebug() << m_listResult;
+    kDebug() << m_displayNameListResult;
     QCOMPARE(m_listResult.count( "." ), 1); // found it, and only once
     QCOMPARE(m_listResult.count("0-fileFromHome"), 1);
     QCOMPARE(m_listResult.count("0-fileFromHome 1"), 1);
@@ -1042,7 +1083,6 @@ void TestTrash::listRecursiveRootDir()
     QCOMPARE(m_displayNameListResult.count("fileFromHome 1"), 1);
     QCOMPARE(m_displayNameListResult.count("trashDirFromHome/testfile"), 1);
     QCOMPARE(m_displayNameListResult.count("readonly/readonly_subdir/testfile_in_subdir"), 1);
-    
 }
 
 void TestTrash::listSubDir()
@@ -1050,16 +1090,16 @@ void TestTrash::listSubDir()
     m_entryCount = 0;
     m_listResult.clear();
     m_displayNameListResult.clear();
-    KIO::ListJob* job = KIO::listDir( QUrl("trash:/0-trashDirFromHome"), KIO::HideProgressInfo );
+    KIO::ListJob* job = KIO::listDir( KUrl("trash:/0-trashDirFromHome"), KIO::HideProgressInfo );
     connect( job, SIGNAL( entries( KIO::Job*, const KIO::UDSEntryList& ) ),
              SLOT( slotEntries( KIO::Job*, const KIO::UDSEntryList& ) ) );
     bool ok = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( ok );
-    qDebug() << "listDir done - m_entryCount=" << m_entryCount;
+    kDebug() << "listDir done - m_entryCount=" << m_entryCount;
     QCOMPARE(m_entryCount, 3);
 
-    //qDebug() << m_listResult;
-    //qDebug() << m_displayNameListResult;
+    //kDebug() << m_listResult;
+    //kDebug() << m_displayNameListResult;
     QCOMPARE(m_listResult.count( "." ), 1); // found it, and only once
     QCOMPARE(m_listResult.count( "testfile" ), 1); // found it, and only once
     QCOMPARE(m_listResult.count("subdir"), 1);
@@ -1073,10 +1113,10 @@ void TestTrash::slotEntries( KIO::Job*, const KIO::UDSEntryList& lst )
         const KIO::UDSEntry& entry (*it);
         QString name = entry.stringValue(KIO::UDSEntry::UDS_NAME);
         QString displayName = entry.stringValue(KIO::UDSEntry::UDS_DISPLAY_NAME);
-        QUrl url(entry.stringValue( KIO::UDSEntry::UDS_URL ));
-        qDebug() << "name" << name << "displayName" << displayName << " UDS_URL=" << url;
+        KUrl url = entry.stringValue( KIO::UDSEntry::UDS_URL );
+        kDebug() << "name" << name << "displayName" << displayName << " UDS_URL=" << url;
         if ( !url.isEmpty() ) {
-            QVERIFY( url.scheme() == "trash" );
+            QVERIFY( url.protocol() == "trash" );
         }
         m_listResult << name;
         m_displayNameListResult << displayName;
@@ -1100,7 +1140,7 @@ void TestTrash::emptyTrash()
     QByteArray packedArgs;
     QDataStream stream( &packedArgs, QIODevice::WriteOnly );
     stream << (int)1;
-    KIO::Job* job = KIO::special( QUrl( "trash:/" ), packedArgs, KIO::HideProgressInfo );
+    KIO::Job* job = KIO::special( KUrl( "trash:/" ), packedArgs, KIO::HideProgressInfo );
     bool ok = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( ok );
 
@@ -1115,28 +1155,28 @@ void TestTrash::emptyTrash()
     QVERIFY(QDir(m_trashDir + "/files").entryList(QDir::NoDotAndDotDot|QDir::AllEntries).isEmpty());
 
 #else
-    qDebug() << " : SKIPPED";
+    kDebug() << " : SKIPPED";
 #endif
 }
 
-void TestTrash::testTrashSize()
+void TestTrash::testEmptyTrashSize()
 {
-    KIO::DirectorySizeJob* job = KIO::directorySize(QUrl("trash:/"));
+    KIO::DirectorySizeJob* job = KIO::directorySize(KUrl("trash:/"));
     QVERIFY(job->exec());
     QVERIFY(job->totalSize() < 1000000000 /*1GB*/); // #157023
 }
 
-static void checkIcon( const QUrl& url, const QString& expectedIcon )
+static void checkIcon( const KUrl& url, const QString& expectedIcon )
 {
-    QString icon = KIO::iconNameForUrl( url );
+    QString icon = KMimeType::iconNameForUrl( url );
     QCOMPARE( icon, expectedIcon );
 }
 
 void TestTrash::testIcons()
 {
     QCOMPARE(KProtocolInfo::icon("trash"), QString("user-trash-full") ); // #100321
-    checkIcon( QUrl("trash:/"), "user-trash-full" ); // #100321
-    checkIcon( QUrl("trash:/foo/"), "inode-directory" );
+    checkIcon( KUrl("trash:/"), "user-trash-full" ); // #100321
+    checkIcon( KUrl("trash:/foo/"), "inode-directory" );
 }
 
 QTEST_MAIN(TestTrash) // QT5 TODO: NOGUI
